CONFIGURATION_DATA = {
    'implemented': ['rgb', 'hex', 'hsl'],
    'maximum_values': {
        'rgb': 255
    },
    'lengths': {
        'rgb': 3,
        'hex': 1,
        'hsl': 3
    },
    'hsl_check_values_ranges': {
        'saturation': {
            'min': 0.05,
            'max': 0.95
        },
        'lightness': {
            'min': 0.05,
            'max': 0.95
        }
    }
}
